# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

"""
Download handling.
"""

from __future__ import annotations

import hashlib
import logging
import time
from dataclasses import dataclass
from pathlib import Path
from typing import BinaryIO

import requests

from license_tools.constants import VERSION

logger = logging.getLogger(__name__)
del logging


USER_AGENT = f"https://github.com/stefan6419846/license_tools version {VERSION}"


class ChecksumError(ValueError):
    """
    Error indicating a wrong checksum.
    """


@dataclass
class Download:
    """
    Configuration for one file download.
    """

    url: str
    """
    The download URL.
    """

    filename: str
    """
    The target filename.
    """

    sha256: str | None = None
    """
    The expected SHA256 checksum.
    """

    def verify_checksum(self, data: bytes) -> None:
        """
        Check if the checksum of the given data matches the expected one.

        Raises :class:`~ChecksumError` if something is wrong.

        :param data: The data to check.
        """
        if self.sha256 is not None:
            digest = hashlib.sha256(data).hexdigest()
            expected = self.sha256
        else:
            return
        if digest != expected:
            raise ChecksumError(f"Checksum mismatch: Got {digest}, expected {expected}!")


class DownloadError(ValueError):
    """
    Error indicating some (generic) download failure.
    """


def get_session() -> requests.Session:
    """
    Get an identifiable session.

    :return: The session which identifies us against the server.
    """
    session = requests.Session()
    session.headers.update({"User-Agent": USER_AGENT})
    return session


def download_file(url: str, file_object: BinaryIO) -> None:
    """
    Download the given file to the given file object.

    :param url: The download URL to use.
    :param file_object: The binary file to download to. Reset after writing.
    """
    session = get_session()
    response = session.get(url)
    if not response.ok:
        raise DownloadError(f"Download not okay? {url} {response}")
    file_object.write(response.content)
    file_object.seek(0)


def download_file_to_directory(download: Download, directory: Path, session: requests.Session | None = None) -> None:
    """
    Download the given file to the given directory.

    :param download: Download to perform.
    :param directory: Directory to download to.
    :param session: Session to use.
    """
    if session is None:
        session = get_session()
    target_path = directory / download.filename
    logger.info("Downloading %s to %s ...", download.url, target_path)
    response = session.get(download.url)
    if not response.ok:
        raise DownloadError(f"Download not okay? {download.url} {response}")
    download.verify_checksum(response.content)
    target_path.write_bytes(response.content)


def download_one_file_per_second(downloads: list[Download], directory: Path) -> None:
    """
    Download the given files with not more than one request per second. This conforms to
    https://crates.io/data-access#api accordingly.

    :param downloads: List of downloads to perform.
    :param directory: Directory to download to.
    """
    session = get_session()
    for download in downloads:
        download_file_to_directory(download=download, directory=directory, session=session)
        time.sleep(1)



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

"""
Retrieve license-related data.
"""

from __future__ import annotations

import atexit
import math
import shutil
from collections import defaultdict
from collections.abc import Generator
from pathlib import Path
from tempfile import NamedTemporaryFile, TemporaryDirectory
from typing import BinaryIO, cast

import scancode_config  # type: ignore[import-untyped]
from joblib import Parallel, delayed  # type: ignore[import-untyped]

from license_tools.tools import cargo_tools, font_tools, image_tools, linking_tools, pip_tools, scancode_tools
from license_tools.tools.pip_tools import download_package
from license_tools.tools.scancode_tools import FileResults, Licenses, PackageResults
from license_tools.utils import archive_utils
from license_tools.utils.download_utils import download_file
from license_tools.utils.path_utils import DirectoryWithFixedNameContext, get_files_from_directory


class RetrievalFlags:
    """
    Data retrieval flags to get shorter parameter lists.
    """

    COPYRIGHTS = 1
    EMAILS = 2
    FILE_INFO = 4
    URLS = 8
    LDD_DATA = 16
    FONT_DATA = 32
    PYTHON_METADATA = 64
    CARGO_METADATA = 128
    IMAGE_METADATA = 256

    @classmethod
    def to_int(
        cls,
        retrieve_copyrights: bool = False,
        retrieve_emails: bool = False,
        retrieve_file_info: bool = False,
        retrieve_urls: bool = False,
        retrieve_ldd_data: bool = False,
        retrieve_font_data: bool = False,
        retrieve_python_metadata: bool = False,
        retrieve_cargo_metadata: bool = False,
        retrieve_image_metadata: bool = False,
    ) -> int:
        """
        Convert the given boolean parameter values to a single integer flag value.

        :param retrieve_copyrights: Whether to retrieve copyright information.
        :param retrieve_emails: Whether to retrieve e-mails.
        :param retrieve_file_info: Whether to retrieve file-specific information.
        :param retrieve_urls: Whether to retrieve URLs.
        :param retrieve_ldd_data: Whether to retrieve linking data for shared objects.
        :param retrieve_font_data: Whether to retrieve font data.
        :param retrieve_python_metadata: Whether to retrieve Python package metadata.
        :param retrieve_cargo_metadata: Whether to retrieve Cargo metadata.
        :param retrieve_image_metadata: Whether to retrieve image metadata.
        :return: The flags derived from the given parameters.
        """
        return (
            cls.COPYRIGHTS * retrieve_copyrights
            + cls.EMAILS * retrieve_emails
            + cls.FILE_INFO * retrieve_file_info
            + cls.URLS * retrieve_urls
            + cls.LDD_DATA * retrieve_ldd_data
            + cls.FONT_DATA * retrieve_font_data
            + cls.PYTHON_METADATA * retrieve_python_metadata
            + cls.CARGO_METADATA * retrieve_cargo_metadata
            + cls.IMAGE_METADATA * retrieve_image_metadata
        )

    @classmethod
    def all(cls, as_kwargs: bool = False) -> int | dict[str, bool]:
        """
        Utility method to enable all flags.

        :param: If enabled, return kwargs instead of the integer value.
        :return: The value for all flags enabled.
        """
        value = cls.to_int(
            retrieve_copyrights=True, retrieve_emails=True, retrieve_file_info=True, retrieve_urls=True, retrieve_ldd_data=True, retrieve_font_data=True,
            retrieve_python_metadata=True, retrieve_cargo_metadata=True, retrieve_image_metadata=True,
        )
        if as_kwargs:
            return cls.to_kwargs(value)
        return value

    @classmethod
    def is_set(cls, flags: int, flag: int) -> bool:
        """
        Check if the given flag is set.

        :param flags: The flags to check inside.
        :param flag: The flag value to check for.
        :return: The check result.
        """
        return bool(flags & flag)

    @classmethod
    def to_kwargs(cls, flags: int) -> dict[str, bool]:
        """
        Convert the given flags to keyword arguments.

        :param flags: The flags to convert.
        :return: The associated keyword arguments.
        """
        return {
            "retrieve_copyrights": cls.is_set(flags=flags, flag=cls.COPYRIGHTS),
            "retrieve_emails": cls.is_set(flags=flags, flag=cls.EMAILS),
            "retrieve_file_info": cls.is_set(flags=flags, flag=cls.FILE_INFO),
            "retrieve_urls": cls.is_set(flags=flags, flag=cls.URLS),
            "retrieve_ldd_data": cls.is_set(flags=flags, flag=cls.LDD_DATA),
            "retrieve_font_data": cls.is_set(flags=flags, flag=cls.FONT_DATA),
            "retrieve_python_metadata": cls.is_set(flags=flags, flag=cls.PYTHON_METADATA),
            "retrieve_cargo_metadata": cls.is_set(flags=flags, flag=cls.CARGO_METADATA),
            "retrieve_image_metadata": cls.is_set(flags=flags, flag=cls.IMAGE_METADATA),
        }


def _run_on_archive_file(path: Path, short_path: str, default_to_none: bool = False) -> FileResults | None:
    """
    Run archive-specific analysis.

    :param path: The path to run on.
    :param short_path: The short path to use for display.
    :param default_to_none: Return `None` if no results could be determined instead of an empty entry.
    :return: Explicitly declared results.
    """
    if path.suffix == ".rpm" or (path.suffixes and ".rpm" in path.suffixes):
        rpm_results = PackageResults.from_rpm(path)
        if rpm_results.declared_license_expression_spdx:
            results = _get_dummy_file_results(path=path, short_path=short_path)
            results.licenses = Licenses(
                detected_license_expression=rpm_results.declared_license_expression_spdx,
                detected_license_expression_spdx=rpm_results.declared_license_expression_spdx,
            )
            return results
    if default_to_none:
        return None
    return _get_dummy_file_results(path=path, short_path=short_path)


def _get_dummy_file_results(
        path: Path,
        short_path: str,
) -> FileResults:
    """
    Get some empty dummy license results, which allows displaying the file in the
    results, but skipping the actual analysis.

    :param path: The file path to analyze.
    :param short_path: The short path to use for display.
    :return: Minimal results.
    """
    results = FileResults(path=path, short_path=short_path)
    results.licenses = Licenses()
    return results


def run_on_file(
    path: Path,
    short_path: str,
    retrieval_flags: int = 0,
) -> FileResults:
    """
    Run the analysis on the given file.

    :param path: The file path to analyze.
    :param short_path: The short path to use for display.
    :param retrieval_flags: Values to retrieve.
    :return: The requested results.
    """
    if archive_utils.can_extract(archive_path=path):
        # Archive files which can be extracted further are not being analyzed on the
        # file level. This should improve the extraction speed and avoid possible
        # memory errors as this skips running string matching on possibly large
        # archives which are binary blobs anyway and do not provide any real value.
        # Instead, just have a quick look at their headers if they provide any useful
        # values.
        return cast(
            FileResults,
            _run_on_archive_file(path=path, short_path=short_path, default_to_none=False),
        )

    retrieval_kwargs = RetrievalFlags.to_kwargs(flags=retrieval_flags)

    if path.name.startswith("Cargo.toml") and retrieval_kwargs.pop("retrieve_cargo_metadata"):
        print(short_path)
        print(cargo_tools.check_metadata(path=path) + "\n")

    # This data is not yet part of the dataclasses above, as it is a custom analysis.
    # Return early if we got a result here, as these binary files currently do not
    # provide any additional useful insights in most of the cases, but tend to be
    # larger binary blobs which just slow down the analysis.
    if retrieval_kwargs.pop("retrieve_ldd_data"):
        results = linking_tools.check_shared_objects(path=path)
        if results:
            print(short_path + "\n" + results)
            return _get_dummy_file_results(path=path, short_path=short_path)
    if retrieval_kwargs.pop("retrieve_font_data"):
        results = font_tools.check_font(path=path)
        if results:
            print(short_path + "\n" + results + "\n")
            return _get_dummy_file_results(path=path, short_path=short_path)
    if retrieval_kwargs.pop("retrieve_image_metadata"):
        results = image_tools.check_image_metadata(path=path)
        if results:
            print(short_path + "\n" + results)
            return _get_dummy_file_results(path=path, short_path=short_path)

    # Register this here as each parallel process has its own directory.
    atexit.register(scancode_tools.cleanup, scancode_config.scancode_temp_dir)

    return FileResults(
        path=path, short_path=short_path, retrieve_licenses=True,
        retrieve_copyrights=retrieval_kwargs["retrieve_copyrights"],
        retrieve_emails=retrieval_kwargs["retrieve_emails"],
        retrieve_urls=retrieval_kwargs["retrieve_urls"],
        retrieve_file_info=retrieval_kwargs["retrieve_file_info"],
    )


def run_on_directory(
    directory: str,
    job_count: int = 4,
    retrieval_flags: int = 0,
    prefix: str | None = None,
    allow_random_directory_for_archive: bool = True,
    delete_unpacked_archive_directories: bool = True,
) -> Generator[FileResults]:
    """
    Run the analysis on the given directory.

    :param directory: The directory to analyze.
    :param job_count: The number of parallel jobs to use.
    :param retrieval_flags: Values to retrieve.
    :param prefix: Custom prefix to use.
    :param allow_random_directory_for_archive: Allow a random directory for unpacking archives.
    :param delete_unpacked_archive_directories: Delete the directories of unpacked archives afterwards.
    :return: The requested results per file.
    """
    files = list(get_files_from_directory(directory, prefix))
    results = Parallel(n_jobs=job_count)(
        delayed(run_on_file)(
            path=path,
            short_path=short_path,
            retrieval_flags=retrieval_flags,
        )
        for path, short_path in files
    )
    yield from results

    for path, _ in files:
        if archive_utils.can_extract(path):
            name = path.name[:-len("".join(path.suffixes))]
            subdirectory = path.parent / f'{name}_{"_".join(path.suffixes).replace(".", "")}'
            with DirectoryWithFixedNameContext(
                    directory=subdirectory,
                    fallback_to_random_if_exists=allow_random_directory_for_archive,
                    delete_afterwards=delete_unpacked_archive_directories,
            ) as target_directory:
                archive_utils.extract(archive_path=path, target_directory=target_directory)
                yield from run_on_directory(
                    directory=str(target_directory),
                    job_count=job_count,
                    retrieval_flags=retrieval_flags,
                    prefix=directory,
                )


def run_on_package_archive_file(
    archive_path: Path,
    job_count: int = 4,
    retrieval_flags: int = 0,
) -> Generator[FileResults]:
    """
    Run the analysis on the given package archive file.

    :param archive_path: The package archive path to analyze.
    :param job_count: The number of parallel jobs to use.
    :param retrieval_flags: Values to retrieve.
    :return: The requested results.
    """
    archive_results = _run_on_archive_file(path=archive_path, short_path=archive_path.name, default_to_none=True)
    if archive_results is not None:
        yield archive_results

    with TemporaryDirectory() as working_directory:
        if not archive_utils.can_extract(archive_path):
            raise ValueError(f"Unsupported archive format: {archive_path}")
        archive_utils.extract(archive_path=archive_path, target_directory=Path(working_directory))

        if RetrievalFlags.is_set(flags=retrieval_flags, flag=RetrievalFlags.PYTHON_METADATA):
            print(pip_tools.check_metadata(path=working_directory) + "\n")

        yield from run_on_directory(
            directory=working_directory,
            job_count=job_count,
            retrieval_flags=retrieval_flags,
        )


def run_on_downloaded_archive_file(
    download_url: str,
    job_count: int = 4,
    retrieval_flags: int = 0,
) -> Generator[FileResults]:
    """
    Run the analysis on the given archive file after downloading it.

    :param download_url: The URL to download the archive from.
    :param job_count: The number of parallel jobs to use.
    :param retrieval_flags: Values to retrieve.
    :return: The requested results.
    """
    # Retrieving the correct suffixes is a bit tricky here, so we use some guessing as well.
    # This basically uses the trailing URL part (usually the filename itself) and forwards
    # guessing the suffixes to Python itself. Due to the way the suffixes are determined,
    # we probably get some part of the (irrelevant) filename as well as the suffix (due to
    # dotted package versions for example), but this should not hurt. We just have to make
    # sure that we do not really loose important suffix information as Python will not be
    # able to unpack this archive otherwise, for example because we supply `.gz` instead
    # of `.tar.gz` only.
    suffixes = Path(download_url.rsplit("/", maxsplit=1)[1]).suffixes
    suffix = "".join(suffixes)
    with NamedTemporaryFile(suffix=suffix) as downloaded_file:
        download_file(url=download_url, file_object=cast(BinaryIO, downloaded_file))
        yield from run_on_package_archive_file(
            archive_path=Path(downloaded_file.name),
            job_count=job_count,
            retrieval_flags=retrieval_flags,
        )


def run_on_downloaded_package_file(
    package_definition: str,
    index_url: str | None = None,
    job_count: int = 4,
    retrieval_flags: int = 0,
    prefer_sdist: bool = False,
) -> Generator[FileResults]:
    """
    Run the analysis for the given package definition.

    :param package_definition: The package definition to get the files for.
    :param index_url: The PyPI index URL to use. Uses the default one from the `.pypirc` file if unset.
    :param job_count: The number of parallel jobs to use.
    :param retrieval_flags: Values to retrieve.
    :param prefer_sdist: Download the source distribution instead of the wheel.
    :return: The requested results.
    """
    with TemporaryDirectory() as download_directory:
        download_package(
            package_definition=package_definition,
            download_directory=download_directory,
            index_url=index_url,
            prefer_sdist=prefer_sdist,
        )
        name = next(Path(download_directory).glob("*"))
        yield from run_on_package_archive_file(
            archive_path=name.resolve(),
            job_count=job_count,
            retrieval_flags=retrieval_flags,
        )


def _check_that_exactly_one_value_is_set(values: list[Path | str | None]) -> bool:
    """
    Check that exactly one value does not evaluate to `False`.
    """
    filtered = list(filter(None, values))
    return len(filtered) == 1


def run(
    directory: Path | str | None = None,
    file_path: Path | str | None = None,
    archive_path: Path | str | None = None,
    package_definition: str | None = None,
    download_url: str | None = None,
    index_url: str | None = None,
    prefer_sdist: bool = False,
    job_count: int = 4,
    retrieve_copyrights: bool = False,
    retrieve_emails: bool = False,
    retrieve_file_info: bool = False,
    retrieve_urls: bool = False,
    retrieve_ldd_data: bool = False,
    retrieve_font_data: bool = False,
    retrieve_python_metadata: bool = False,
    retrieve_cargo_metadata: bool = False,
    retrieve_image_metadata: bool = False,
) -> list[FileResults]:
    """
    Run the analysis for the given input definition.

    The `directory`, `file_path`, `archive_path` and `package_definition` parameters are mutually exclusive,
    but exactly one has to be set.

    :param directory: The directory to run on.
    :param file_path: The file to run on.
    :param archive_path: The package archive to run on.
    :param package_definition: The package definition to run for.
    :param download_url: The package URL to download and run on.
    :param index_url: The PyPI index URL to use. Uses the default one from the `.pypirc` file if unset.
    :param prefer_sdist: For PyPI downloads, prefer/use the source distribution over/instead of the wheel.
    :param job_count: The number of parallel jobs to use.
    :param retrieve_copyrights: Whether to retrieve copyright information.
    :param retrieve_emails: Whether to retrieve e-mails.
    :param retrieve_file_info: Whether to retrieve file-specific information.
    :param retrieve_urls: Whether to retrieve URLs.
    :param retrieve_ldd_data: Whether to retrieve linking data for shared objects.
    :param retrieve_font_data: Whether to retrieve font data.
    :param retrieve_python_metadata: Whether to retrieve Python package metadata.
    :param retrieve_cargo_metadata: Whether to retrieve Cargo metadata.
    :param retrieve_image_metadata: Whether to retrieve image metadata.
    :return: The requested results.
    """
    # Remove the temporary directory of the main thread.
    atexit.register(scancode_tools.cleanup, scancode_config.scancode_temp_dir)

    assert _check_that_exactly_one_value_is_set(
        [directory, file_path, archive_path, package_definition, download_url],
    ), "Exactly one source is required."

    license_counts: dict[str | None, int] = defaultdict(int)
    retrieval_flags = RetrievalFlags.to_int(
        retrieve_copyrights=retrieve_copyrights,
        retrieve_emails=retrieve_emails,
        retrieve_file_info=retrieve_file_info,
        retrieve_urls=retrieve_urls,
        retrieve_ldd_data=retrieve_ldd_data,
        retrieve_font_data=retrieve_font_data,
        retrieve_python_metadata=bool(retrieve_python_metadata and package_definition),
        retrieve_cargo_metadata=retrieve_cargo_metadata,
        retrieve_image_metadata=retrieve_image_metadata,
    )

    # Run the analysis itself.
    if package_definition:
        results = list(
            run_on_downloaded_package_file(
                package_definition=package_definition,
                index_url=index_url,
                retrieval_flags=retrieval_flags,
                job_count=job_count,
                prefer_sdist=prefer_sdist,
            ),
        )
    elif directory:
        results = list(
            run_on_directory(
                directory=str(directory),
                retrieval_flags=retrieval_flags,
                job_count=job_count,
            ),
        )
    elif archive_path:
        results = list(
            run_on_package_archive_file(
                archive_path=Path(archive_path),
                retrieval_flags=retrieval_flags,
                job_count=job_count,
            ),
        )
    elif download_url:
        results = list(
            run_on_downloaded_archive_file(
                download_url=download_url,
                retrieval_flags=retrieval_flags,
                job_count=job_count,
            ),
        )
    elif file_path:
        results = [
            run_on_file(
                path=Path(file_path),
                short_path=str(file_path),
                retrieval_flags=retrieval_flags,
            ),
         ]
    else:
        return []

    # Display the file-level results.
    max_path_length = max(len(result.short_path) for result in results)
    for result in results:
        if result.licenses is None:
            continue
        licenses = result.licenses
        scores = licenses.get_scores_of_detected_license_expression_spdx()
        print(
            f"{result.short_path:>{max_path_length}}",
            f"{licenses.detected_license_expression_spdx:>70}"
            if licenses.detected_license_expression_spdx
            else " " * 70,
            scores if scores else "",
        )
        license_counts[licenses.detected_license_expression_spdx] += 1

    # Display the license-level results.
    print()
    columns = shutil.get_terminal_size((80, 20)).columns
    print("=" * columns)
    print()
    count_length = max(math.log10(count) for count in license_counts.values())
    count_length = int(count_length) + 1
    for identifier in sorted(license_counts, key=str):
        print(
            f"{identifier!s:>70}", f"{license_counts[identifier]:>{count_length + 1}d}",
        )

    return results


atexit.register(scancode_tools.cleanup, scancode_config.scancode_temp_dir)



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

from __future__ import annotations

import re
from typing import cast
from unittest import TestCase

from license_tools.tools import image_tools
from tests import get_file
from tests.data import LICENSE_PATH, SETUP_PATH


class IsImageTestCase(TestCase):
    def test_is_image(self) -> None:
        self.assertFalse(image_tools.is_image(SETUP_PATH))
        self.assertFalse(image_tools.is_image(LICENSE_PATH))

        with get_file("Carlito-Regular.ttf") as path:
            self.assertFalse(image_tools.is_image(path))

        with get_file("croissant.jpg") as path:
            self.assertTrue(image_tools.is_image(path))


class CheckImageMetadataTestCase(TestCase):
    maxDiff = None

    def test_python(self) -> None:
        self.assertIsNone(image_tools.check_image_metadata(SETUP_PATH))

    def test_font(self) -> None:
        with get_file("Carlito-Regular.ttf") as path:
            self.assertIsNone(image_tools.check_image_metadata(path))

    def assert_regex(self, expected_regex: str, text: str) -> None:
        # Like `assertRegex`, but using `match` instead of `search` and
        # providing the actual text as failure message.
        if not re.match(expected_regex, text):
            self.fail(text)

    def test_croissant_jpg(self) -> None:
        with get_file("croissant.jpg") as path:
            result = image_tools.check_image_metadata(path)

        self.assertIsNotNone(result)

        self.assert_regex(
            expected_regex=r"""
^\[ExifTool\]      ExifTool Version Number         : \d+\.\d+
\[File\]          File Name                       : croissant\.jpg
\[File\]          File Size                       : (184 kB|180 KiB)
\[File\]          File Modification Date/Time     : \d{4}:\d{2}:\d{2} \d{2}:\d{2}:\d{2}\+\d{2}:\d{2}
\[File\]          File Permissions                : \-rw\-r\-\-r\-\-
\[File\]          File Type                       : JPEG
\[File\]          File Type Extension             : jpg
\[File\]          MIME Type                       : image/jpeg
\[File\]          Exif Byte Order                 : Little\-endian \(Intel, II\)
\[File\]          Current IPTC Digest             : fce11f89c8b7c9782f346234075877eb
\[File\]          Image Width                     : 1280
\[File\]          Image Height                    : 853
\[File\]          Encoding Process                : Baseline DCT, Huffman coding
\[File\]          Bits Per Sample                 : 8
\[File\]          Color Components                : 3
\[File\]          Y Cb Cr Sub Sampling            : YCbCr4:4:4 \(1 1\)
\[Ducky\]         Quality                         : 70%
\[XMP\]           XMP Toolkit                     : Adobe XMP Core 5\.0\-c060 61\.134777, 2010/02/12\-17:32:00
\[XMP\]           Marked                          : False
\[XMP\]           Web Statement                   : http://www\.publicdomainpictures\.net
\[XMP\]           Original Document ID            : xmp\.did:D61CE24A664AE011B7F5DC66CB8B70E4
\[XMP\]           Document ID                     : xmp\.did:13A209884A6E11E09364B31F7A5A30E9
\[XMP\]           Instance ID                     : xmp\.iid:13A209874A6E11E09364B31F7A5A30E9
\[XMP\]           Creator Tool                    : Adobe Photoshop CS5 Windows
\[XMP\]           Derived From Instance ID        : xmp\.iid:EE2C2F47694AE011B7F5DC66CB8B70E4
\[XMP\]           Derived From Document ID        : xmp\.did:D61CE24A664AE011B7F5DC66CB8B70E4
\[XMP\]           Creator                         : Petr Kratochvil
\[XMP\]           Title                           : croissant
\[IPTC\]          Coded Character Set             : UTF8
\[IPTC\]          Application Record Version      : 2
\[Photoshop\]     IPTC Digest                     : fce11f89c8b7c9782f346234075877eb
\[ICC_Profile\]   Profile CMM Type                : Linotronic
\[ICC_Profile\]   Profile Version                 : 2\.1\.0
\[ICC_Profile\]   Profile Class                   : Display Device Profile
\[ICC_Profile\]   Color Space Data                : RGB
\[ICC_Profile\]   Profile Connection Space        : XYZ
\[ICC_Profile\]   Profile Date Time               : 1998:02:09 06:49:00
\[ICC_Profile\]   Profile File Signature          : acsp
\[ICC_Profile\]   Primary Platform                : Microsoft Corporation
\[ICC_Profile\]   CMM Flags                       : Not Embedded, Independent
\[ICC_Profile\]   Device Manufacturer             : Hewlett\-Packard
\[ICC_Profile\]   Device Model                    : sRGB
\[ICC_Profile\]   Device Attributes               : Reflective, Glossy, Positive, Color
\[ICC_Profile\]   Rendering Intent                : Perceptual
\[ICC_Profile\]   Connection Space Illuminant     : 0\.9642 1 0\.82491
\[ICC_Profile\]   Profile Creator                 : Hewlett\-Packard
\[ICC_Profile\]   Profile ID                      : 0
\[ICC_Profile\]   Profile Copyright               : Copyright \(c\) 1998 Hewlett\-Packard Company
\[ICC_Profile\]   Profile Description             : sRGB IEC61966\-2\.1
\[ICC_Profile\]   Media White Point               : 0\.95045 1 1\.08905
\[ICC_Profile\]   Media Black Point               : 0 0 0
\[ICC_Profile\]   Red Matrix Column               : 0\.43607 0\.22249 0\.01392
\[ICC_Profile\]   Green Matrix Column             : 0\.38515 0\.71687 0\.09708
\[ICC_Profile\]   Blue Matrix Column              : 0\.14307 0\.06061 0\.7141
\[ICC_Profile\]   Device Mfg Desc                 : IEC http://www\.iec\.ch
\[ICC_Profile\]   Device Model Desc               : IEC 61966\-2\.1 Default RGB colour space \- sRGB
\[ICC_Profile\]   Viewing Cond Desc               : Reference Viewing Condition in IEC61966\-2\.1
\[ICC_Profile\]   Viewing Cond Illuminant         : 19\.6445 20\.3718 16\.8089
\[ICC_Profile\]   Viewing Cond Surround           : 3\.92889 4\.07439 3\.36179
\[ICC_Profile\]   Viewing Cond Illuminant Type    : D50
\[ICC_Profile\]   Luminance                       : 76\.03647 80 87\.12462
\[ICC_Profile\]   Measurement Observer            : CIE 1931
\[ICC_Profile\]   Measurement Backing             : 0 0 0
\[ICC_Profile\]   Measurement Geometry            : Unknown
\[ICC_Profile\]   Measurement Flare               : 0\.999%
\[ICC_Profile\]   Measurement Illuminant          : D65
\[ICC_Profile\]   Technology                      : Cathode Ray Tube Display
\[ICC_Profile\]   Red Tone Reproduction Curve     : \(Binary data 2060 bytes, use \-b option to extract\)
\[ICC_Profile\]   Green Tone Reproduction Curve   : \(Binary data 2060 bytes, use \-b option to extract\)
\[ICC_Profile\]   Blue Tone Reproduction Curve    : \(Binary data 2060 bytes, use \-b option to extract\)
\[APP14\]         DCT Encode Version              : 100
\[APP14\]         APP14 Flags 0                   : \[14\], Encoded with Blend=1 downsampling
\[APP14\]         APP14 Flags 1                   : \(none\)
\[APP14\]         Color Transform                 : YCbCr
\[Composite\]     Image Size                      : 1280x853
\[Composite\]     Megapixels                      : 1\.1
            """.strip() + "\n$",
            text=cast(str, result),
        )

    def test_mountain_jpg(self) -> None:
        with get_file("mountain.jpg") as path:
            result = image_tools.check_image_metadata(path)

        self.assertIsNotNone(result)

        self.assert_regex(
            expected_regex=r"""
^\[ExifTool\]      ExifTool Version Number         : \d+\.\d+
\[ExifTool\]      Warning                         : \[minor\] Adjusted MakerNotes base by \-108
\[File\]          File Name                       : mountain\.jpg
\[File\]          File Size                       : (424 kB|414 KiB)
\[File\]          File Modification Date/Time     : \d{4}:\d{2}:\d{2} \d{2}:\d{2}:\d{2}\+\d{2}:\d{2}
\[File\]          File Permissions                : \-rw\-r\-\-r\-\-
\[File\]          File Type                       : JPEG
\[File\]          File Type Extension             : jpg
\[File\]          MIME Type                       : image/jpeg
\[File\]          Current IPTC Digest             : d41d8cd98f00b204e9800998ecf8427e
\[File\]          Exif Byte Order                 : Big\-endian \(Motorola, MM\)
\[File\]          Image Width                     : 630
\[File\]          Image Height                    : 420
\[File\]          Encoding Process                : Baseline DCT, Huffman coding
\[File\]          Bits Per Sample                 : 8
\[File\]          Color Components                : 3
\[File\]          Y Cb Cr Sub Sampling            : YCbCr4:4:4 \(1 1\)
\[JFIF\]          JFIF Version                    : 1\.01
\[JFIF\]          Resolution Unit                 : inches
\[JFIF\]          X Resolution                    : 180
\[JFIF\]          Y Resolution                    : 180
\[Photoshop\]     IPTC Digest                     : d41d8cd98f00b204e9800998ecf8427e
\[EXIF\]          Image Description               : 
\[EXIF\]          Make                            : Canon
\[EXIF\]          Camera Model Name               : Canon PowerShot G7 X
\[EXIF\]          Orientation                     : Horizontal \(normal\)
\[EXIF\]          X Resolution                    : 180
\[EXIF\]          Y Resolution                    : 180
\[EXIF\]          Resolution Unit                 : inches
\[EXIF\]          Software                        : Camera\+ 8\.0\.2
\[EXIF\]          Modify Date                     : 2016:06:17 15:15:38
\[EXIF\]          Artist                          : 
\[EXIF\]          Host Computer                   : iPhone \(iPhone OS 9\.3\.5\)
\[EXIF\]          Y Cb Cr Positioning             : Centered
\[EXIF\]          Copyright                       : 
\[EXIF\]          Exposure Time                   : 1/1250
\[EXIF\]          F Number                        : 4\.5
\[EXIF\]          ISO                             : 250
\[EXIF\]          Sensitivity Type                : Recommended Exposure Index
\[EXIF\]          Recommended Exposure Index      : 250
\[EXIF\]          Exif Version                    : 0230
\[EXIF\]          Date/Time Original              : 2016:06:17 15:15:38
\[EXIF\]          Create Date                     : 2016:06:17 15:15:38
\[EXIF\]          Components Configuration        : Y, Cb, Cr, \-
\[EXIF\]          Shutter Speed Value             : 1/1244
\[EXIF\]          Aperture Value                  : 4\.5
\[EXIF\]          Exposure Compensation           : 0
\[EXIF\]          Max Aperture Value              : 2\.0
\[EXIF\]          Metering Mode                   : Multi\-segment
\[EXIF\]          Flash                           : Off, Did not fire
\[EXIF\]          Focal Length                    : 10\.3 mm
\[EXIF\]          User Comment                    : 
\[EXIF\]          Flashpix Version                : 0100
\[EXIF\]          Color Space                     : sRGB
\[EXIF\]          Exif Image Width                : 3420
\[EXIF\]          Exif Image Height               : 2280
\[EXIF\]          Focal Plane X Resolution        : 10584\.13793
\[EXIF\]          Focal Plane Y Resolution        : 10573\.91304
\[EXIF\]          Focal Plane Resolution Unit     : inches
\[EXIF\]          Sensing Method                  : One\-chip color area
\[EXIF\]          File Source                     : Digital Camera
\[EXIF\]          Custom Rendered                 : Normal
\[EXIF\]          Exposure Mode                   : Auto
\[EXIF\]          White Balance                   : Auto
\[EXIF\]          Digital Zoom Ratio              : 1
\[EXIF\]          Scene Capture Type              : Standard
\[EXIF\]          Owner Name                      : 
\[EXIF\]          Compression                     : JPEG \(old\-style\)
\[EXIF\]          X Resolution                    : 72
\[EXIF\]          Y Resolution                    : 72
\[EXIF\]          Resolution Unit                 : inches
\[EXIF\]          Thumbnail Offset                : 6928
\[EXIF\]          Thumbnail Length                : 10506
\[EXIF\]          Thumbnail Image                 : \(Binary data 10506 bytes, use \-b option to extract\)
\[MakerNotes\]    Macro Mode                      : Normal
\[MakerNotes\]    Self Timer                      : Off
\[MakerNotes\]    Quality                         : Fine
\[MakerNotes\]    Canon Flash Mode                : Off
\[MakerNotes\]    Continuous Drive                : Single
\[MakerNotes\]    Focus Mode                      : Single
\[MakerNotes\]    Record Mode                     : JPEG
\[MakerNotes\]    Canon Image Size                : Large
\[MakerNotes\]    Easy Mode                       : Full auto
\[MakerNotes\]    Digital Zoom                    : None
\[MakerNotes\]    Contrast                        : Normal
\[MakerNotes\]    Saturation                      : Normal
\[MakerNotes\]    Sharpness                       : 0
\[MakerNotes\]    Camera ISO                      : Auto
\[MakerNotes\]    Metering Mode                   : Evaluative
\[MakerNotes\]    Focus Range                     : Auto
\[MakerNotes\]    AF Point                        : Face Detect
\[MakerNotes\]    Canon Exposure Mode             : Easy
\[MakerNotes\]    Lens Type                       : n/a
\[MakerNotes\]    Max Focal Length                : 36\.8 mm
\[MakerNotes\]    Min Focal Length                : 8\.8 mm
\[MakerNotes\]    Focal Units                     : 1000/mm
\[MakerNotes\]    Max Aperture                    : 2
\[MakerNotes\]    Min Aperture                    : 11
\[MakerNotes\]    Flash Bits                      : \(none\)
\[MakerNotes\]    Focus Continuous                : Continuous
\[MakerNotes\]    AE Setting                      : Normal AE
\[MakerNotes\]    Image Stabilization             : On \(2\)
\[MakerNotes\]    Zoom Source Width               : 5472
\[MakerNotes\]    Zoom Target Width               : 5472
\[MakerNotes\]    Spot Metering Mode              : Center
\[MakerNotes\]    Manual Flash Output             : n/a
\[MakerNotes\]    Auto ISO                        : 116
\[MakerNotes\]    Base ISO                        : 200
\[MakerNotes\]    Measured EV                     : 14\.31
\[MakerNotes\]    Target Aperture                 : 4\.5
\[MakerNotes\]    Target Exposure Time            : 1/1244
\[MakerNotes\]    Exposure Compensation           : 0
\[MakerNotes\]    White Balance                   : Auto
\[MakerNotes\]    Slow Shutter                    : Off
\[MakerNotes\]    Shot Number In Continuous Burst : 0
\[MakerNotes\]    Optical Zoom Code               : 9
\[MakerNotes\]    Flash Guide Number              : 0
\[MakerNotes\]    Flash Exposure Compensation     : 0
\[MakerNotes\]    Auto Exposure Bracketing        : Off
\[MakerNotes\]    AEB Bracket Value               : 0
\[MakerNotes\]    Control Mode                    : Camera Local Control
\[MakerNotes\]    Focus Distance Upper            : 65\.53 m
\[MakerNotes\]    Focus Distance Lower            : 0 m
\[MakerNotes\]    F Number                        : 4\.8
\[MakerNotes\]    Exposure Time                   : 1/1448
\[MakerNotes\]    Bulb Duration                   : 0
\[MakerNotes\]    Camera Type                     : Compact
\[MakerNotes\]    Auto Rotate                     : None
\[MakerNotes\]    ND Filter                       : Off
\[MakerNotes\]    Self Timer 2                    : 0
\[MakerNotes\]    Flash Output                    : 0
\[MakerNotes\]    Canon Image Type                : IMG:PowerShot G7 X JPEG
\[MakerNotes\]    Canon Firmware Version          : Firmware Version 1\.00
\[MakerNotes\]    File Number                     : 118\-4673
\[MakerNotes\]    Camera Temperature              : 27 C
\[MakerNotes\]    Canon Model ID                  : PowerShot G7 X
\[MakerNotes\]    Thumbnail Image Valid Area      : 0 159 7 112
\[MakerNotes\]    Date Stamp Mode                 : Off
\[MakerNotes\]    My Color Mode                   : Off
\[MakerNotes\]    Firmware Revision               : 1\.00 rev 4\.00
\[MakerNotes\]    Categories                      : \(none\)
\[MakerNotes\]    Intelligent Contrast            : Off
\[MakerNotes\]    Image Unique ID                 : 24175e01993d4a6282f6143d3085d4d7
\[MakerNotes\]    Faces Detected                  : 0
\[MakerNotes\]    Time Zone                       : \+00:00
\[MakerNotes\]    Time Zone City                  : \(not set\)
\[MakerNotes\]    Daylight Savings                : Off
\[MakerNotes\]    AF Area Mode                    : Auto
\[MakerNotes\]    Num AF Points                   : 35
\[MakerNotes\]    Valid AF Points                 : 31
\[MakerNotes\]    Canon Image Width               : 5472
\[MakerNotes\]    Canon Image Height              : 3648
\[MakerNotes\]    AF Image Width                  : 100
\[MakerNotes\]    AF Image Height                 : 100
\[MakerNotes\]    AF Area Widths                  : 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12 12
\[MakerNotes\]    AF Area Heights                 : 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18 18
\[MakerNotes\]    AF Area X Positions             : \-24 \-12 0 12 24 \-36 \-24 \-12 0 12 24 36 \-36 \-24 \-12 0 12 24 36 \-36 \-24 \-12 0 12 24 36 \-24 \-12 0 12 24 0 12 24 36
\[MakerNotes\]    AF Area Y Positions             : \-36 \-36 \-36 \-36 \-36 \-18 \-18 \-18 \-18 \-18 \-18 \-18 0 0 0 0 0 0 0 18 18 18 18 18 18 18 36 36 36 36 36 36 36 36 36
\[MakerNotes\]    AF Points In Focus              : 6,7,12,13,14,19,20
\[MakerNotes\]    Aspect Ratio                    : 3:2
\[MakerNotes\]    Cropped Image Width             : 5472
\[MakerNotes\]    Cropped Image Height            : 3648
\[MakerNotes\]    Cropped Image Left              : 0
\[MakerNotes\]    Cropped Image Top               : 0
\[MakerNotes\]    VRD Offset                      : 0
\[Composite\]     Drive Mode                      : Single\-frame Shooting
\[Composite\]     ISO                             : 233
\[Composite\]     Lens                            : 8\.8 \- 36\.8 mm
\[Composite\]     Shooting Mode                   : Full auto
\[Composite\]     Aperture                        : 4\.5
\[Composite\]     Image Size                      : 630x420
\[Composite\]     Lens ID                         : Unknown 8\-36mm
\[Composite\]     Megapixels                      : 0\.265
\[Composite\]     Scale Factor To 35 mm Equivalent: 4\.4
\[Composite\]     Shutter Speed                   : 1/1250
\[Composite\]     Lens                            : 8\.8 \- 36\.8 mm \(35 mm equivalent: 38\.6 \- 161\.4 mm\)
\[Composite\]     Circle Of Confusion             : 0\.007 mm
\[Composite\]     Depth Of Field                  : inf \(3\.12 m \- inf\)
\[Composite\]     Field Of View                   : 43\.4 deg
\[Composite\]     Focal Length                    : 10\.3 mm \(35 mm equivalent: 45\.2 mm\)
\[Composite\]     Hyperfocal Distance             : 3\.45 m
\[Composite\]     Light Value                     : 13\.3
            """.strip() + "\n$",  # noqa: E501, W291
            text=cast(str, result),
        )



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

from __future__ import annotations

import shutil
from pathlib import Path
from tempfile import NamedTemporaryFile, TemporaryDirectory
from unittest import TestCase

from license_tools.utils import archive_utils
from license_tools.utils.path_utils import get_files_from_directory
from tests import get_from_url
from tests.data import (
    BASE64__0_22_0__CRATE,
    JSON__20231013__JAR,
    LIBAIO1__0_3_109_1_25__RPM,
    LIBAIO1__0_3_109_1_25__SRC_RPM,
    TYPING_EXTENSION_4_8_0__SOURCE_FILES,
    TYPING_EXTENSION_4_8_0__WHEEL_FILES,
    TYPING_EXTENSIONS__4_8_0__SDIST,
    TYPING_EXTENSIONS__4_8_0__WHEEL,
)


class ArchiveUtilsTestCase(TestCase):
    def test_jar(self) -> None:
        with get_from_url(JSON__20231013__JAR) as path, TemporaryDirectory() as tempdir:
            directory = Path(tempdir)
            self.assertTrue(archive_utils.can_extract(path))
            archive_utils.extract(archive_path=path, target_directory=directory)
            actual = [x[1] for x in get_files_from_directory(directory)]
            self.assertEqual(
                [
                    "META-INF/MANIFEST.MF",
                    "META-INF/maven/org.json/json/pom.properties",
                    "META-INF/maven/org.json/json/pom.xml",
                    "org/json/CDL.class",
                    "org/json/Cookie.class",
                    "org/json/CookieList.class",
                    "org/json/HTTP.class",
                    "org/json/HTTPTokener.class",
                    "org/json/JSONArray.class",
                    "org/json/JSONException.class",
                    "org/json/JSONML.class",
                    "org/json/JSONMLParserConfiguration.class",
                    "org/json/JSONObject$1.class",
                    "org/json/JSONObject$Null.class",
                    "org/json/JSONObject.class",
                    "org/json/JSONPointer$Builder.class",
                    "org/json/JSONPointer.class",
                    "org/json/JSONPointerException.class",
                    "org/json/JSONPropertyIgnore.class",
                    "org/json/JSONPropertyName.class",
                    "org/json/JSONString.class",
                    "org/json/JSONStringer.class",
                    "org/json/JSONTokener.class",
                    "org/json/JSONWriter.class",
                    "org/json/ParserConfiguration.class",
                    "org/json/Property.class",
                    "org/json/XML$1$1.class",
                    "org/json/XML$1.class",
                    "org/json/XML.class",
                    "org/json/XMLParserConfiguration.class",
                    "org/json/XMLTokener.class",
                    "org/json/XMLXsiTypeConverter.class",
                ],
                actual,
            )

    def test_wheel(self) -> None:
        with get_from_url(TYPING_EXTENSIONS__4_8_0__WHEEL) as path, TemporaryDirectory() as tempdir:
            directory = Path(tempdir)
            self.assertTrue(archive_utils.can_extract(path))
            archive_utils.extract(archive_path=path, target_directory=directory)
            actual = [x[1] for x in get_files_from_directory(directory)]
            self.assertEqual(TYPING_EXTENSION_4_8_0__WHEEL_FILES, actual)

    def test_rpm_file(self) -> None:
        with get_from_url(LIBAIO1__0_3_109_1_25__RPM) as path, TemporaryDirectory() as tempdir:
            directory = Path(tempdir)
            self.assertTrue(archive_utils.can_extract(path))
            archive_utils.extract(archive_path=path, target_directory=directory)
            actual = [x[1] for x in get_files_from_directory(directory)]
            self.assertEqual(
                [
                    "lib64/libaio.so.1.0.1",
                    "usr/share/doc/packages/libaio1/COPYING",
                    "usr/share/doc/packages/libaio1/TODO",
                ],
                actual,
            )

    def test_tar_gz(self) -> None:
        with get_from_url(TYPING_EXTENSIONS__4_8_0__SDIST) as path, TemporaryDirectory() as tempdir:
            directory = Path(tempdir)
            self.assertTrue(archive_utils.can_extract(path))
            archive_utils.extract(archive_path=path, target_directory=directory)
            actual = [x[1] for x in get_files_from_directory(directory)]
            self.assertEqual(TYPING_EXTENSION_4_8_0__SOURCE_FILES, actual)

    def test_rust_crate(self) -> None:
        with get_from_url(BASE64__0_22_0__CRATE) as path, TemporaryDirectory() as tempdir:
            directory = Path(tempdir)
            self.assertTrue(archive_utils.can_extract(path))
            archive_utils.extract(archive_path=path, target_directory=directory)
            actual = [x[1] for x in get_files_from_directory(directory)]
            self.assertEqual(
                [
                    "base64-0.22.0/.cargo_vcs_info.json",
                    "base64-0.22.0/.circleci/config.yml",
                    "base64-0.22.0/.github/ISSUE_TEMPLATE/general-purpose-issue.md",
                    "base64-0.22.0/.gitignore",
                    "base64-0.22.0/Cargo.lock",
                    "base64-0.22.0/Cargo.toml",
                    "base64-0.22.0/Cargo.toml.orig",
                    "base64-0.22.0/LICENSE-APACHE",
                    "base64-0.22.0/LICENSE-MIT",
                    "base64-0.22.0/README.md",
                    "base64-0.22.0/RELEASE-NOTES.md",
                    "base64-0.22.0/benches/benchmarks.rs",
                    "base64-0.22.0/clippy.toml",
                    "base64-0.22.0/examples/base64.rs",
                    "base64-0.22.0/icon_CLion.svg",
                    "base64-0.22.0/src/alphabet.rs",
                    "base64-0.22.0/src/chunked_encoder.rs",
                    "base64-0.22.0/src/decode.rs",
                    "base64-0.22.0/src/display.rs",
                    "base64-0.22.0/src/encode.rs",
                    "base64-0.22.0/src/engine/general_purpose/decode.rs",
                    "base64-0.22.0/src/engine/general_purpose/decode_suffix.rs",
                    "base64-0.22.0/src/engine/general_purpose/mod.rs",
                    "base64-0.22.0/src/engine/mod.rs",
                    "base64-0.22.0/src/engine/naive.rs",
                    "base64-0.22.0/src/engine/tests.rs",
                    "base64-0.22.0/src/lib.rs",
                    "base64-0.22.0/src/prelude.rs",
                    "base64-0.22.0/src/read/decoder.rs",
                    "base64-0.22.0/src/read/decoder_tests.rs",
                    "base64-0.22.0/src/read/mod.rs",
                    "base64-0.22.0/src/tests.rs",
                    "base64-0.22.0/src/write/encoder.rs",
                    "base64-0.22.0/src/write/encoder_string_writer.rs",
                    "base64-0.22.0/src/write/encoder_tests.rs",
                    "base64-0.22.0/src/write/mod.rs",
                    "base64-0.22.0/tests/encode.rs",
                    "base64-0.22.0/tests/tests.rs",
                ],
                actual,
            )

    def test_zip(self) -> None:
        with TemporaryDirectory() as source, TemporaryDirectory() as tempdir, NamedTemporaryFile() as zip_file:
            zip_path = Path(f"{zip_file.name}.zip")
            self.addCleanup(zip_path.unlink)
            source_path = Path(source)
            directory = Path(tempdir)
            source_path.joinpath("test.txt").write_text("abc")
            source_path.joinpath("directory1").mkdir()
            source_path.joinpath("directory2").mkdir()
            source_path.joinpath("directory2", "README").write_text("Hello World")
            shutil.make_archive(
                base_name=zip_file.name,
                format="zip",
                root_dir=source_path.parent,
                base_dir=source_path.name,
            )

            self.assertTrue(archive_utils.can_extract(zip_path))
            archive_utils.extract(archive_path=zip_path, target_directory=directory)
            actual = [x[1] for x in get_files_from_directory(directory)]
            self.assertEqual(
                [
                    f"{source_path.name}/directory2/README",
                    f"{source_path.name}/test.txt",
                ],
                actual,
            )
            self.assertEqual(
                {"directory1", "directory2", "test.txt"},
                {x.name for x in directory.joinpath(source_path.name).glob("*")},
            )

    def test_unknown(self) -> None:
        self.assertFalse(archive_utils.can_extract(Path("/home/bin/run.exe")))

    def test_nested(self) -> None:
        with get_from_url(LIBAIO1__0_3_109_1_25__SRC_RPM) as path:
            with TemporaryDirectory() as tempdir:
                directory = Path(tempdir)
                self.assertTrue(archive_utils.can_extract(path))
                archive_utils.extract(
                    archive_path=path,
                    target_directory=directory,
                    recurse=True,
                )
                actual = [x[1] for x in get_files_from_directory(directory)]
                self.assertEqual(
                    [
                        "00_arches.patch",
                        "00_arches_sh.patch",
                        "01_link_libgcc.patch",
                        "02_libdevdir.patch",
                        "03_man_errors.patch",
                        "baselibs.conf",
                        "libaio-0.3.109.tar.bz2",
                        "libaio-0.3.109/.version",
                        "libaio-0.3.109/COPYING",
                        "libaio-0.3.109/ChangeLog",
                        "libaio-0.3.109/INSTALL",
                        "libaio-0.3.109/Makefile",
                        "libaio-0.3.109/TODO",
                        "libaio-0.3.109/harness/Makefile",
                        "libaio-0.3.109/harness/README",
                        "libaio-0.3.109/harness/attic/0.t",
                        "libaio-0.3.109/harness/attic/1.t",
                        "libaio-0.3.109/harness/cases/10.t",
                        "libaio-0.3.109/harness/cases/11.t",
                        "libaio-0.3.109/harness/cases/12.t",
                        "libaio-0.3.109/harness/cases/13.t",
                        "libaio-0.3.109/harness/cases/14.t",
                        "libaio-0.3.109/harness/cases/15.t",
                        "libaio-0.3.109/harness/cases/16.t",
                        "libaio-0.3.109/harness/cases/2.t",
                        "libaio-0.3.109/harness/cases/3.t",
                        "libaio-0.3.109/harness/cases/4.t",
                        "libaio-0.3.109/harness/cases/5.t",
                        "libaio-0.3.109/harness/cases/6.t",
                        "libaio-0.3.109/harness/cases/7.t",
                        "libaio-0.3.109/harness/cases/8.t",
                        "libaio-0.3.109/harness/cases/aio_setup.h",
                        "libaio-0.3.109/harness/cases/common-7-8.h",
                        "libaio-0.3.109/harness/ext2-enospc.img",
                        "libaio-0.3.109/harness/main.c",
                        "libaio-0.3.109/harness/runtests.sh",
                        "libaio-0.3.109/libaio.spec",
                        "libaio-0.3.109/man/aio.3",
                        "libaio-0.3.109/man/aio_cancel.3",
                        "libaio-0.3.109/man/aio_cancel64.3",
                        "libaio-0.3.109/man/aio_error.3",
                        "libaio-0.3.109/man/aio_error64.3",
                        "libaio-0.3.109/man/aio_fsync.3",
                        "libaio-0.3.109/man/aio_fsync64.3",
                        "libaio-0.3.109/man/aio_init.3",
                        "libaio-0.3.109/man/aio_read.3",
                        "libaio-0.3.109/man/aio_read64.3",
                        "libaio-0.3.109/man/aio_return.3",
                        "libaio-0.3.109/man/aio_return64.3",
                        "libaio-0.3.109/man/aio_suspend.3",
                        "libaio-0.3.109/man/aio_suspend64.3",
                        "libaio-0.3.109/man/aio_write.3",
                        "libaio-0.3.109/man/aio_write64.3",
                        "libaio-0.3.109/man/io.3",
                        "libaio-0.3.109/man/io_cancel.1",
                        "libaio-0.3.109/man/io_cancel.3",
                        "libaio-0.3.109/man/io_destroy.1",
                        "libaio-0.3.109/man/io_fsync.3",
                        "libaio-0.3.109/man/io_getevents.1",
                        "libaio-0.3.109/man/io_getevents.3",
                        "libaio-0.3.109/man/io_prep_fsync.3",
                        "libaio-0.3.109/man/io_prep_pread.3",
                        "libaio-0.3.109/man/io_prep_pwrite.3",
                        "libaio-0.3.109/man/io_queue_init.3",
                        "libaio-0.3.109/man/io_queue_release.3",
                        "libaio-0.3.109/man/io_queue_run.3",
                        "libaio-0.3.109/man/io_queue_wait.3",
                        "libaio-0.3.109/man/io_set_callback.3",
                        "libaio-0.3.109/man/io_setup.1",
                        "libaio-0.3.109/man/io_submit.1",
                        "libaio-0.3.109/man/io_submit.3",
                        "libaio-0.3.109/man/lio_listio.3",
                        "libaio-0.3.109/man/lio_listio64.3",
                        "libaio-0.3.109/src/Makefile",
                        "libaio-0.3.109/src/compat-0_1.c",
                        "libaio-0.3.109/src/io_cancel.c",
                        "libaio-0.3.109/src/io_destroy.c",
                        "libaio-0.3.109/src/io_getevents.c",
                        "libaio-0.3.109/src/io_queue_init.c",
                        "libaio-0.3.109/src/io_queue_release.c",
                        "libaio-0.3.109/src/io_queue_run.c",
                        "libaio-0.3.109/src/io_queue_wait.c",
                        "libaio-0.3.109/src/io_setup.c",
                        "libaio-0.3.109/src/io_submit.c",
                        "libaio-0.3.109/src/libaio.h",
                        "libaio-0.3.109/src/libaio.map",
                        "libaio-0.3.109/src/raw_syscall.c",
                        "libaio-0.3.109/src/syscall-alpha.h",
                        "libaio-0.3.109/src/syscall-arm.h",
                        "libaio-0.3.109/src/syscall-i386.h",
                        "libaio-0.3.109/src/syscall-ia64.h",
                        "libaio-0.3.109/src/syscall-ppc.h",
                        "libaio-0.3.109/src/syscall-s390.h",
                        "libaio-0.3.109/src/syscall-x86_64.h",
                        "libaio-0.3.109/src/syscall.h",
                        "libaio-0.3.109/src/vsys_def.h",
                        "libaio-aarch64-support.diff",
                        "libaio-generic-arch.diff",
                        "libaio-optflags.diff",
                        "libaio.spec",
                    ],
                    actual,
                )

            with TemporaryDirectory() as tempdir, TemporaryDirectory() as tempdir2:
                directory = Path(tempdir)
                tar_bz2_directory = Path(tempdir2)
                self.assertTrue(archive_utils.can_extract(path))
                archive_utils.extract(
                    archive_path=path,
                    target_directory=directory,
                    recurse=False,
                )
                actual = [x[1] for x in get_files_from_directory(directory)]
                self.assertEqual(
                    [
                        "00_arches.patch",
                        "00_arches_sh.patch",
                        "01_link_libgcc.patch",
                        "02_libdevdir.patch",
                        "03_man_errors.patch",
                        "baselibs.conf",
                        "libaio-0.3.109.tar.bz2",
                        "libaio-aarch64-support.diff",
                        "libaio-generic-arch.diff",
                        "libaio-optflags.diff",
                        "libaio.spec",
                    ],
                    actual,
                )

                archive_utils.extract(
                    archive_path=directory / "libaio-0.3.109.tar.bz2",
                    target_directory=tar_bz2_directory,
                    recurse=False,
                )
                actual = [
                    x[1] for x in get_files_from_directory(tar_bz2_directory)
                ]
                self.assertEqual(
                    [
                        "libaio-0.3.109/.version",
                        "libaio-0.3.109/COPYING",
                        "libaio-0.3.109/ChangeLog",
                        "libaio-0.3.109/INSTALL",
                        "libaio-0.3.109/Makefile",
                        "libaio-0.3.109/TODO",
                        "libaio-0.3.109/harness/Makefile",
                        "libaio-0.3.109/harness/README",
                        "libaio-0.3.109/harness/attic/0.t",
                        "libaio-0.3.109/harness/attic/1.t",
                        "libaio-0.3.109/harness/cases/10.t",
                        "libaio-0.3.109/harness/cases/11.t",
                        "libaio-0.3.109/harness/cases/12.t",
                        "libaio-0.3.109/harness/cases/13.t",
                        "libaio-0.3.109/harness/cases/14.t",
                        "libaio-0.3.109/harness/cases/15.t",
                        "libaio-0.3.109/harness/cases/16.t",
                        "libaio-0.3.109/harness/cases/2.t",
                        "libaio-0.3.109/harness/cases/3.t",
                        "libaio-0.3.109/harness/cases/4.t",
                        "libaio-0.3.109/harness/cases/5.t",
                        "libaio-0.3.109/harness/cases/6.t",
                        "libaio-0.3.109/harness/cases/7.t",
                        "libaio-0.3.109/harness/cases/8.t",
                        "libaio-0.3.109/harness/cases/aio_setup.h",
                        "libaio-0.3.109/harness/cases/common-7-8.h",
                        "libaio-0.3.109/harness/ext2-enospc.img",
                        "libaio-0.3.109/harness/main.c",
                        "libaio-0.3.109/harness/runtests.sh",
                        "libaio-0.3.109/libaio.spec",
                        "libaio-0.3.109/man/aio.3",
                        "libaio-0.3.109/man/aio_cancel.3",
                        "libaio-0.3.109/man/aio_cancel64.3",
                        "libaio-0.3.109/man/aio_error.3",
                        "libaio-0.3.109/man/aio_error64.3",
                        "libaio-0.3.109/man/aio_fsync.3",
                        "libaio-0.3.109/man/aio_fsync64.3",
                        "libaio-0.3.109/man/aio_init.3",
                        "libaio-0.3.109/man/aio_read.3",
                        "libaio-0.3.109/man/aio_read64.3",
                        "libaio-0.3.109/man/aio_return.3",
                        "libaio-0.3.109/man/aio_return64.3",
                        "libaio-0.3.109/man/aio_suspend.3",
                        "libaio-0.3.109/man/aio_suspend64.3",
                        "libaio-0.3.109/man/aio_write.3",
                        "libaio-0.3.109/man/aio_write64.3",
                        "libaio-0.3.109/man/io.3",
                        "libaio-0.3.109/man/io_cancel.1",
                        "libaio-0.3.109/man/io_cancel.3",
                        "libaio-0.3.109/man/io_destroy.1",
                        "libaio-0.3.109/man/io_fsync.3",
                        "libaio-0.3.109/man/io_getevents.1",
                        "libaio-0.3.109/man/io_getevents.3",
                        "libaio-0.3.109/man/io_prep_fsync.3",
                        "libaio-0.3.109/man/io_prep_pread.3",
                        "libaio-0.3.109/man/io_prep_pwrite.3",
                        "libaio-0.3.109/man/io_queue_init.3",
                        "libaio-0.3.109/man/io_queue_release.3",
                        "libaio-0.3.109/man/io_queue_run.3",
                        "libaio-0.3.109/man/io_queue_wait.3",
                        "libaio-0.3.109/man/io_set_callback.3",
                        "libaio-0.3.109/man/io_setup.1",
                        "libaio-0.3.109/man/io_submit.1",
                        "libaio-0.3.109/man/io_submit.3",
                        "libaio-0.3.109/man/lio_listio.3",
                        "libaio-0.3.109/man/lio_listio64.3",
                        "libaio-0.3.109/src/Makefile",
                        "libaio-0.3.109/src/compat-0_1.c",
                        "libaio-0.3.109/src/io_cancel.c",
                        "libaio-0.3.109/src/io_destroy.c",
                        "libaio-0.3.109/src/io_getevents.c",
                        "libaio-0.3.109/src/io_queue_init.c",
                        "libaio-0.3.109/src/io_queue_release.c",
                        "libaio-0.3.109/src/io_queue_run.c",
                        "libaio-0.3.109/src/io_queue_wait.c",
                        "libaio-0.3.109/src/io_setup.c",
                        "libaio-0.3.109/src/io_submit.c",
                        "libaio-0.3.109/src/libaio.h",
                        "libaio-0.3.109/src/libaio.map",
                        "libaio-0.3.109/src/raw_syscall.c",
                        "libaio-0.3.109/src/syscall-alpha.h",
                        "libaio-0.3.109/src/syscall-arm.h",
                        "libaio-0.3.109/src/syscall-i386.h",
                        "libaio-0.3.109/src/syscall-ia64.h",
                        "libaio-0.3.109/src/syscall-ppc.h",
                        "libaio-0.3.109/src/syscall-s390.h",
                        "libaio-0.3.109/src/syscall-x86_64.h",
                        "libaio-0.3.109/src/syscall.h",
                        "libaio-0.3.109/src/vsys_def.h",
                    ],
                    actual,
                )



# Copyright (c) stefan6419846. All rights reserved.
# SPDX-License-Identifier: Apache-2.0
# See http://www.apache.org/licenses/LICENSE-2.0 for the license text.

"""
Tools related to translations.
"""

from __future__ import annotations

import logging
import shutil
import subprocess
from pathlib import Path

from license_tools.utils.path_utils import get_mime_type

logger = logging.getLogger(__name__)
del logging


def is_compiled_gettext_file(path: Path) -> bool:
    """
    Check if the given file is a compiled gettext translation file.

    :param path: The file to check.
    :return: True if the file is a compiled gettext translation file, False otherwise.
    """
    mime_type = get_mime_type(path).lower()
    return mime_type.startswith("application/x-gettext-translation")


def check_compiled_gettext_metadata(path: Path) -> str | None:
    """
    Check the metadata of the given compiled gettext translation file.

    :param path: The file path to analyze.
    :return: The analysis results if the path points to a compiled gettext translation file, `None` otherwise.
    """
    if not is_compiled_gettext_file(path):
        return None

    # Ideally, we would have used https://github.com/izimobil/polib here instead of doing subprocess calls,
    # but the package does not really appear to be maintained enough to consider it as a reliable backend.
    msgunfmt = shutil.which("msgunfmt")
    if not msgunfmt:
        raise ValueError("msgunfmt not found!")

    output = subprocess.check_output(
        [
            msgunfmt,
            "--force-po",
            path,
        ],
        stderr=subprocess.PIPE,
    )
    return output.decode("UTF-8")
